#ifdef GL_ES
precision mediump float;
#endif

uniform sampler2D tex0;
varying vec2 tcoord;

// http://empire-defense.crystalin.fr/blog/2d_shock_wave_texture_with_shader
// http://www.geeks3d.com/20091116/shader-library-2d-shockwave-post-processing-filter-glsl/

uniform vec2 u_resolution;

uniform vec2 u_pos[16];
uniform float u_time[16];
uniform float u_scale[16];

uniform float u_amplitude[16]; 	// 10
uniform float u_refraction[16];	// 0.8
uniform float u_width[16];     	// 0.1


vec2 ripple(vec2 pos, float amplitude, float refraction, float width, float time, float scale, float invAr) {

	vec2 uv = vec2(tcoord.x / scale, (tcoord.y * invAr) / scale);
	vec2 texCoord = tcoord;

	vec2 center = vec2(pos.x / u_resolution.x / scale, (pos.y * invAr) / u_resolution.y / scale);
	float dist = distance(uv, center);

	if ((dist <= (time + width)) && (dist >= (time - width))) {
		
		float diff = (dist - time);
		float powDiff = 1.0 - pow(abs(diff * amplitude), refraction);
		float diffTime = diff  * powDiff;
		vec2 diffUV = normalize(uv - center);

		return (diffUV * diffTime);
	}

	return vec2(0,0);

}


void main() {

	float invAr = u_resolution.y / u_resolution.x;
	vec2 texCoord = tcoord;

	for(int i = 0; i < 16; i++) {

		if(u_time[i] > 0.0){
			texCoord += ripple(u_pos[i], u_amplitude[i], u_refraction[i], u_width[i], u_time[i], u_scale[i], invAr);
		}

	}

	gl_FragColor = texture2D(tex0, texCoord);

}
