

import clay.Parcel;
import clay.ParcelProgress;
import clay.resource.Resource;

import clay.GameConfig;
// import clay.IComponent;
import clay.Visual;
import clay.NodeList;
import clay.Input;
import clay.Entity;
import clay.Color;
import clay.Vector;
import clay.fsm.EntityStateMachine;
import clay.components.SpriteComponent;
import clay.components.GameObjectComponent;
import clay.components.ParticleSystem;
import clay.components.TransformComponent;
// import components.physics.Collider;
// import components.physics.Gravity;
import effects.Effects;
import effects.BloomEffect;
import effects.SepiaEffect;
import effects.ShockwaveEffect;

import phoenix.Batcher;
import phoenix.Camera;


class Main extends clay.Game {


	public static var hudBatcher:Batcher;
	public static var effects:Effects;
	public static var shockwave:ShockwaveEffect;

	// var fps:utils.FPS;

	override function ready() {

		Clay.fixed_timestep = true;
		Clay.fixed_frame_time = 1/60;

		clay.macros.Validator.validateJson('assets/parcel.json');
		Clay.resources.load_json('assets/parcel.json').then(function(json:JSONResource) {
			var parcel = new Parcel();
			parcel.from_json(json.asset.json);

			var progress = new ParcelProgress({
				parcel      : parcel,
				background  : new Color(1,1,1,0.85),
				oncomplete  : assetsLoaded
			});
			parcel.load();
		});


	} //ready

	function assetsLoaded(_) {

		hudBatcher = new Batcher(Clay.renderer, 'hud batcher');
		var hud_view = new Camera();
		hudBatcher.view = hud_view;
		hudBatcher.layer = 2;
		Clay.renderer.add_batch(hudBatcher);

		// fps = new utils.FPS({
		// 	name : 'fps',
		// 	name_unique : false,
		// 	batcher : hudBatcher,
		// 	color : new Color(1,1,1,1)
		// 	});

		Clay.renderer.clear_color = Colors.CADET_BLUE;

		effects = new Effects();
		var bloom:BloomEffect = new BloomEffect();
		// var sepia:SepiaEffect = new SepiaEffect();
		shockwave = new ShockwaveEffect();
		effects.addEffect(bloom);
		// effects.addEffect(sepia);
		effects.addEffect(shockwave);

		initNodes();
		initProcessors();

		Spawn.water();
		Spawn.player(new Vector(256, 128));


		// Spawn.player(Clay.screen.mid);
		// var ix:Int = 0;
		// var iy:Int = 0;
		// for (x in 0...30) {
		// 	for (y in 0...20) {
		// 		_entityArray.push(Spawn.tile(new Vector(x*32, y*32)));
		// 	}
		// }


		// _entity = Spawn.tile(new Vector(256, 512));
		// Spawn.tile(new Vector(256, 512));
		// Spawn.tile(new Vector(256+32, 512));
		// Spawn.tile(new Vector(256+32+32, 512));

	}

	function initNodes() {

		// Clay.world.nodes.add(new NodeList(nodes.GravityNode));
		// Clay.world.nodes.add(new NodeList(nodes.ColliderNode));
		// Clay.world.nodes.add(new NodeList(nodes.ColliderTransformNode));
		// Clay.world.nodes.add(new NodeList(nodes.ActorNode));
		Clay.world.nodes.add(new NodeList(nodes.WaterNode));
		Clay.world.nodes.add(new NodeList(nodes.SurfboardNode));
		Clay.world.nodes.add(new NodeList(nodes.PlayerNode));
		
	}

	function initProcessors() {
		// Clay.world.processors.add(new processors.PlayerInputProcessor(), 0);
		// Clay.world.processors.add(new processors.AiInputProcessor(), 1);
		// Clay.world.processors.add(new processors.ActorProcessor(), 2);


		// Clay.world.processors.add(new processors.GravityProcessor(), 3);
		// Clay.world.processors.add(new processors.MovementProcessor(), 4);
		// Clay.world.processors.add(new processors.JumpProcessor(), 5);

		Clay.world.processors.add(new processors.BroadPhaseProcessor(null,null, new Vector(32,Clay.screen.h)), 7);
		// Clay.world.processors.add(new processors.PhysicsProcessor(), 8);
		Clay.world.processors.add(new processors.WaterProcessor(), 8);

		// Clay.world.processors.add(new processors.PositionProcessor(), 10);

		Physics.init();

	}

	override function onMouseMove(event:MouseEvent) {

	}

	override function onMouseUp(event:MouseEvent) {

	}

	override function onKeyUp( e:KeyEvent ) {

		if(e.keycode == Key.escape) {
			Clay.shutdown();
		}

		if(e.keycode == Key.key_z) {
			Clay.camera.zoom -= 0.1;
			if(Clay.camera.zoom <= 0) {
				Clay.camera.zoom = 0.1;
			}
		}

		if(e.keycode == Key.key_x) {
			Clay.camera.zoom += 0.1;
		}

	} //onkeyup

	override function onPreRender() {

	}

	override function onPostRender() {

	}

	override function update(dt:Float) {

	} //update

	override function config(config:GameConfig) {

		config.window.title = 'clay game';
		config.window.width = 960;
		config.window.height = 640;
		config.window.fullscreen = false;

		config.preload.textures.push({ id:'assets/flame.png' });
		config.preload.textures.push({ id:'assets/smoke.png' });
		config.preload.textures.push({ id:'assets/embers.png' });

		config.preload.sounds = [
			{ id:'assets/jump.ogg', is_stream:false },
			{ id:'assets/space_sound.ogg', is_stream:false }
		];

		return config;

	} //config


} //Main

