package components.water;


import clay.IComponent;
import physics.PointMass;
import physics.Spring;
import physics.PointMassList;
import clay.utils.Maths;
import phoenix.geometry.QuadGeometry;


class WaterComponent implements IComponent {


	public var gravity:Float;
	public var baseDamping(default, set):Float;
	public var baseStiffness(default, set):Float;
	public var springDamping(default, set):Float;
	public var springStiffness(default, set):Float;
	public var distance:Int;

	public var pointMasses:PointMassList;
	public var geometry:List<WaterGeometry>;
	// public var springs:Array<Spring>;
	public var sinIdx:Int = 0;
	public var sinPos:Float = 0;
	public var sinOffset:Float = 0;
	public var spawnOffset:Float = 0;

	public function new() {

		gravity = 16;

		baseDamping = 0.5;
		baseStiffness = 10;

		springDamping = 0.6;
		springStiffness = 100;

		distance = 16; // 16

		pointMasses = new PointMassList();
		geometry = new List();
		// springs = [];

	}

	inline function set_baseDamping(value:Float):Float {

		return baseDamping = Maths.clamp(value, 0, 1);

	}

	inline function set_baseStiffness(value:Float):Float {

		return baseStiffness = Maths.clamp_bottom(value, 0);

	}

	inline function set_springDamping(value:Float):Float {

		return springDamping = Maths.clamp(value, 0, 1);

	}

	inline function set_springStiffness(value:Float):Float {

		return springStiffness = Maths.clamp_bottom(value, 0);

	}

}

class WaterGeometry {

	public var pA:PointMass;
	public var pB:PointMass;
	public var geometry:QuadGeometry;

	public function new() {
		
	}

}

