package effects;


import clay.Color;
import clay.Vector;

import phoenix.Batcher;
import phoenix.Shader;
import phoenix.RenderTexture;
import phoenix.geometry.QuadGeometry;

import effects.Effect;


class BloomEffect extends Effect {


	@:isVar public var threshold(default, set):Float;
	@:isVar public var radius(default, set):Float;

	var bloomBrightShader:Shader;
	var bloomBlurShader:Shader;

	var brightBatcher:Batcher;
	var postBrightTexture:RenderTexture;
	var brightGeometry:QuadGeometry;

	var horizBlurBatcher:Batcher;
	var postHorizBlurTexture:RenderTexture;
	var horizBlurGeometry:QuadGeometry;

	var vertBlurBatcher:Batcher;
	var vertBlurGeometry:QuadGeometry;


	public function new() {

		super('bloom');

	}

	override public function onload() {

		// BEGIN SHADERS
		bloomBrightShader = Clay.resources.shader('isolate_bright');
		bloomBrightShader.set_float('brightPassThreshold', 0.5);

		bloomBlurShader = Clay.resources.shader('blur');
		bloomBlurShader.set_float('blur', 1 / Math.min(Clay.screen.w, Clay.screen.h));
		bloomBlurShader.set_vector2('dir', new Vector(1, 0));
		// END SHADERS

		// BEGIN BRIGHTNESS CLAMPER
		brightBatcher = Clay.renderer.create_batcher({
			name: 'batcher.effect.bloom.bright',
			no_add: true
		});
		brightBatcher.view.viewport = Clay.camera.viewport;

		var effectSize = Clay.screen.size;

		brightGeometry = new QuadGeometry({
			x: 0,
			y: 0,
			w:effectSize.x,
			h:effectSize.y,
			batcher: brightBatcher,
			shader: bloomBrightShader
		});

		postBrightTexture = new RenderTexture({ id: 'postBrightTexture', width: Math.floor(effectSize.x), height: Math.floor(effectSize.y) });
		// END BRIGHTNESS CLAMPER

		// BEGIN HORIZONTAL BLUR
		horizBlurBatcher = Clay.renderer.create_batcher({
			name: 'batcher.effect.bloom.hblur',
			no_add: true
		});
		horizBlurBatcher.view.viewport = Clay.camera.viewport;

		horizBlurGeometry = new QuadGeometry({
			texture: postBrightTexture,
			x: 0,
			y: 0,
			w:effectSize.x,
			h:effectSize.y,
			batcher: horizBlurBatcher,
			shader: bloomBlurShader
		});

		postHorizBlurTexture = new RenderTexture({ id: 'postHorizBlurTexture', width: Math.floor(effectSize.x), height: Math.floor(effectSize.y) });
		// END HORIZONTAL BLUR

		// BEGIN VERTICAL BLUR
		vertBlurBatcher = Clay.renderer.create_batcher({
			name: 'batcher.effect.bloom.vblur',
			no_add: true
		});
		vertBlurBatcher.view.viewport = Clay.camera.viewport;

		vertBlurGeometry = new QuadGeometry({
			texture: postHorizBlurTexture,
			x: 0,
			y: 0,
			w:effectSize.x,
			h:effectSize.y,
			batcher: vertBlurBatcher,
			shader: bloomBlurShader
		});
		// END VERTICAL BLUR

		setupGeometry();

		// set the default uniform values
		radius = 2;
		threshold = 0.5;

	}

	override public function apply(preTexture:RenderTexture, postTexture:RenderTexture) {

		// brightness pass
		Clay.renderer.target = postBrightTexture;
		Clay.renderer.clear(clearColour);
		brightGeometry.texture = preTexture;
		brightBatcher.draw();

		// hblur pass
		Clay.renderer.target = postHorizBlurTexture;
		Clay.renderer.clear(clearColour);
		// set the blur direction for the shader
		bloomBlurShader.set_vector2('dir', new Vector(1, 0));
		horizBlurBatcher.draw();

		// draw the incoming texture normally
		Clay.renderer.target = postTexture;
		Clay.renderer.clear(clearColour);
		outputGeometry.texture = preTexture;
		outputBatcher.draw();

		// additive blend the final effect
		Clay.renderer.blend_mode(BlendMode.src_alpha, BlendMode.one);

		// set the blur shader that this is using to blur in the vertical direction
		// so that we get a nice 2D gaussian blur
		bloomBlurShader.set_vector2('dir', new Vector(0, 1));
		vertBlurBatcher.draw();

		// return to default blending
		Clay.renderer.blend_mode();

	}

	function set_threshold(_t:Float) {

		if(bloomBrightShader != null) {
			bloomBrightShader.set_float('brightPassThreshold', _t);
		}

		return threshold = _t;

	}

	function set_radius(_r:Float) {

		if(bloomBlurShader != null) {
			bloomBlurShader.set_float('blur', _r / Math.min(Clay.screen.w, Clay.screen.h));
		}

		return radius = _r;

	}


}
