package effects;


import clay.Vector;

import phoenix.RenderTexture;
import phoenix.Batcher;
import phoenix.geometry.QuadGeometry;

import effects.Effect;


class Effects {


	var effects:Array<EffectData>;
	var preEffectTexture:RenderTexture;
	var screenBatcher:Batcher;
	var screenGeometry:QuadGeometry;
	var loaded:Bool;


	public function new() {

		effects = [];
		loaded = false;

	}

	public function onload() {

		preEffectTexture = new RenderTexture({
			id: "__effects_pre_texture",
			width: Clay.screen.w,
			height: Clay.screen.h
		});

		screenBatcher = Clay.renderer.create_batcher({
			name: 'screenBatcher',
			no_add: true
		});
		screenBatcher.view.viewport = Clay.camera.viewport;

		screenGeometry = new QuadGeometry({
			x:0,
			y:0,
			w:Clay.screen.w,
			h:Clay.screen.h,
			batcher : screenBatcher
		});

		for(e in effects) {
			e.effect.onload();
		}

		loaded = true;
	}

	public function setFlip(flip:Bool) {

		if(flip) {
			screenGeometry.flipy = true;
		} else {
			screenGeometry.flipy = false;
		}

	}

	public function addEffect(effect:Effect) {

		effects.push({
			effect: effect,
			postTexture: new RenderTexture({
				id: "__effects_post_texture." + effects.length,
				width: Clay.screen.w,
				height: Clay.screen.h
			})
		});

		if(loaded) {
			effect.onload();
			setFlip(effects.length % 2 == 0);
		}

	}

	public function update(dt:Float) {

		if(!loaded) {
			return;
		}

		for(e in effects) {
			e.effect.update(dt);
		}

	}

	public function onPreRender() {

		if(!loaded) {
			return;
		}

		Clay.renderer.target = preEffectTexture;

	}

	public function onPostRender() {

		if(!loaded) {
			return;
		}

		var lastTexture:RenderTexture = preEffectTexture;
		for(e in effects) {
			e.effect.apply(lastTexture, e.postTexture);
			lastTexture = e.postTexture;
		}

		Clay.renderer.target = null;
		screenGeometry.texture = lastTexture;
		screenBatcher.draw();

	}


}

typedef EffectData = {
	var effect:Effect;
	var postTexture:RenderTexture;
}
