package effects;


import clay.Color;
import clay.Vector;
import clay.utils.Maths;

import phoenix.Batcher;
import phoenix.Shader;
import phoenix.RenderTexture;

import effects.Effect;

using clay.tweenxcore.Tools;


class ShockwaveEffect extends Effect {


	public var active : Bool = true;

	var waveId :Int = 0;
	var waves :Array<Wave>;
	var shader :Shader;


	public function new() {

		super('ShockwaveEffect');

		shader = Clay.resources.shader('shockwave');

		shader.set_vector2('u_resolution', new Vector(Clay.screen.w, Clay.screen.h));

		waves = [];

		for (i in 0...16) {
			waves[i] = new Wave(i, shader);
		}

	}

	override public function onload() {

		setupGeometry();
		outputGeometry.shader = shader;

	}

	override public function update(dt :Float) {

		if(!active) {
			return;
		}

		for (w in waves) {
			w.update(dt);
		}

	}

	public function wave(_pos:Vector, _time:Float = 1, _speed:Float = 1, _scale:Float = 1, _amplitude:Float = 10, _refraction:Float = 0.8, _width:Float = 0.05):Void {

		if(!active) {
			return;
		}

		waveId++;
		waveId %= 16;

		waves[waveId].doWave(_pos, _time, _speed, _scale, _amplitude, _refraction, _width);

	}

}

private class Wave {


	var id(default, null) : Int;

	var active : Bool = false;

	var effectTimer : Float = 0;
	var effectRate : Float = 0;
	var effectTime : Float = 0;
	var speed : Float = 0;

	var amplitude : Float = 10;
	var refraction : Float = 0.8;
	var width : Float = 0.1;
	var pos : Vector;

	var shader : Shader;

	// names
	var timeName:String;
	var posName:String;
	var scaleName:String;
	var amplitudeName:String;
	var refractionName:String;
	var widthName:String;


	public function new(_id:Int, _shader:Shader) {

		id = _id;
		shader = _shader;

		pos = new Vector();

		timeName = 'u_time[$id]';
		posName = 'u_pos[$id]';
		scaleName = 'u_scale[$id]';
		amplitudeName = 'u_amplitude[$id]';
		refractionName = 'u_refraction[$id]';
		widthName = 'u_width[$id]';

		shader.set_float(timeName, 0.0);
		shader.set_vector2(posName, pos);
		shader.set_float(scaleName, 1);

		shader.set_float(amplitudeName, amplitude);
		shader.set_float(refractionName, refraction);
		shader.set_float(widthName, width);

	}

	public inline function doWave(_pos:Vector, _time:Float = 1, _speed:Float = 1, _scale:Float = 1, _amplitude:Float = 10, _refraction:Float = 0.8, _width:Float = 0.05):Void {

		pos.x = _pos.x;
		pos.y = Clay.screen.height - _pos.y;

		speed = _speed;
		amplitude = _amplitude;
		refraction = _refraction;
		width = _width;

		shader.set_float(timeName, 0.0);
		shader.set_vector2(posName, pos);
		shader.set_float(scaleName, _scale);

		shader.set_float(amplitudeName, amplitude);
		shader.set_float(refractionName, refraction);
		shader.set_float(widthName, width);

		effectTimer = 0;
		effectTime = _time;
		active = true;

	}

	public inline function update(dt:Float) {

		if(active) {

			var dts:Float = dt * Clay.timescale;

			effectRate = effectTimer / effectTime;
			effectTimer += dts;

			if(effectRate < 1) {

				var refr:Float = effectRate.quadOut().lerp(refraction, 0);
				shader.set_float(refractionName, refr);

				var et:Float = effectRate.quadOut().lerp(0, effectTime * speed);
				shader.set_float(timeName, et);

			} else {

				shader.set_float(refractionName, 0);
				shader.set_float(timeName, 0);

				active = false;

			}

		}

	}

}
