package physics;


import clay.Vector;
import physics.PointMass;
import utils.ShapeDrawer;


class Contact {


	public var id:Int;
	/* shape1 */
	public var pmA:PointMass;
	public var pmB1:PointMass;
	public var pmB2:PointMass;
	/* The separation amount */
	public var separation:Float = 0.0;
	/* normal */
	public var normal:Vector;
	/* relative velocity */
	public var velocity:Vector;
	/* The point of contact between the two shapes. */
	public var point:Vector;
	/* first time of collision */
	public var time:Float = 0.0;
	/* last time of collision */
	public var timeLast:Float = 1.0;
	/* should be removed */
	public var remove:Bool = false;

	public var isCollide:Bool = false;
	
	public var wasCollide:Bool = false;


	public inline function new() {

		normal = new Vector();
		point = new Vector();
		velocity = new Vector();

	}

	public inline function clone(){

		var c:Contact = new Contact();
		
		c.pmA = pmA;
		c.pmB1 = pmB1;
		c.pmB2 = pmB2;
		c.separation = separation;
		c.normal = normal.clone();
		c.velocity = velocity.clone();
		c.point = point.clone();
		c.time = time;
		c.timeLast = timeLast;
		c.remove = remove;
		c.isCollide = isCollide;
		c.wasCollide = wasCollide;

		return c;

	}

	public inline function reset(){

		separation = 0;

		normal.x = 0;
		normal.y = 0;

		point.x = 0;
		point.y = 0;

		time = 0;
		timeLast = 1;
		
		return this;

	}

	public inline function clear(){

		reset();

		id = -1;
		remove = false;
		isCollide = false;
		wasCollide = false;

		pmA = null;
		pmB1 = null;
		pmB2 = null;


		return this;

	}

	public inline function destroy():Void{
		
		id = -1;
		
		pmA = null;
		pmB1 = null;
		pmB2 = null;

		point = null;
		normal = null;

	}


	public function draw(immediate:Bool = true){
/*
		if(isCollide){
			ShapeDrawer.drawLine(collider.position, otherCollider.position, ShapeDrawer.color_blue, immediate);
		} else {
			ShapeDrawer.drawLine(collider.position, otherCollider.position, ShapeDrawer.color_gray, immediate);
		}*/
		
		// ShapeDrawer.drawBox_MinMax(min, max, ShapeDrawer.color_gray, immediate);  

		// if(!isStatic){
		//     // ShapeDrawer.drawBox(nextPos, half, ShapeDrawer.color_yellow);
		//     ShapeDrawer.drawLine(position, Vector.Multiply(velocity, Luxe.physics.step_delta).add(position), ShapeDrawer.color_green, immediate);
		// }
	}

	inline function toString() {

		return 
		"pmA: " + pmA.id 
		+ "\npmB1: " + pmB1.id 
		+ "\npmB2: " + pmB2.id 
		+ "\npenetration: " + separation 
		+ "\nnormal: " + normal 
		+ "\npoint: " + point 
		+ "\ntime: " + time
		+ "\ntimeLast: " + timeLast
		;

	}


}
