package physics;

import clay.Vector;
import utils.ShapeDrawer;


class PointMass {

	static var ID:Int = 0;


	public var id:Int;

	public var gridIndex:Array<Int>;
	public var type:Int = 0;

	public var swapped:Bool = false;
	// Global base position of the PointMass.
	public var basePosition:Vector;
	// prev base position of the PointMass.
	public var lastBasePosition:Vector;
	public var offset:Float = 0;

	// Global position of the PointMass.
	public var position:Vector;
	// Global velocity of the PointMass.
	public var velocity:Vector;
    // Force accumulation variable.  Reset to Zero after each call to PointMass.integrate().
	public var force:Vector;
	// External force accumulation variable. Reset only once per FixedUpdate as opposed to once per solver iteration.
	public var externalForce:Vector;
	// Force accumulation variable that is never resety by the simulation, only by the user.
	public var persistantForce:Vector;
	// The shape matching forces calculated for this PointMass will be directly multiplied by this value. 
	public var shapeMatchingMultiplier:Float;
	// mass of thie PointMass.
	public var mass(default, set):Float;
	// The inverse of PointMass.mMass.
	public var invMass(default, null):Float;

	// speed limit
	public var maxSpeed:Float;
	var sqrMaxSpeed:Float;
	var sqrVelocity:Float;

	@:noCompletion public var next:PointMass;
	@:noCompletion public var prev:PointMass;


	public function new() {

		id = ID++;
		gridIndex = [];

		basePosition = new Vector();
		lastBasePosition = new Vector();
		position = new Vector();
		velocity = new Vector();
		force = new Vector();
		externalForce = new Vector();
		persistantForce = new Vector();

		shapeMatchingMultiplier = 1;
		mass = 1;

		maxSpeed = 40;
		sqrMaxSpeed = 0;
		sqrVelocity = 0;

	}

	inline function set_mass(_value:Float):Float{

		if(_value > 0 ){
			mass = _value;
			invMass = 1.0 / mass;
		} else {
			mass = 0;
			invMass = 0;
		}

		return mass;

	}


}
