package physics;

import clay.Vector;

/** Ray intersection data obtained by testing two rays for intersection. */
@:publicFields
class RayIntersection {

    var centroid1:Vector;

    var centroid2:Vector;

    var fraction1:Float = 0.0;

    var fraction2:Float = 0.0;

    var distance1:Float = 0.0;

    var distance2:Float = 0.0;

    var normal:Vector;
    
    var point:Vector;

    @:noCompletion
    inline function new() {
        centroid1 = new Vector();
        centroid2 = new Vector();
        normal = new Vector();
        point = new Vector();
    }

    inline function reset() {

        centroid1.x = 0;
        centroid1.y = 0;

        centroid2.x = 0;
        centroid2.y = 0;

        normal.x = 0;
        normal.y = 0;

        point.x = 0;
        point.y = 0;

        fraction1 = 0;
        fraction2 = 0;
        
        distance1 = 0;
        distance2 = 0;

        return this;

    } //reset

    inline function copy_from(other:RayIntersection) {

        centroid1.x = other.centroid1.x;
        centroid1.y = other.centroid1.y;

        centroid2.x = other.centroid2.x;
        centroid2.y = other.centroid2.y;
        
        point.x = other.point.x;
        point.y = other.point.y;

        normal.x = other.normal.x;
        normal.y = other.normal.y;

        fraction1 = other.fraction1;
        fraction2 = other.fraction2;

        distance1 = other.distance1;
        distance2 = other.distance2;

    } //copy_from

    inline function clone() {

        var _clone = new RayIntersection();

        _clone.copy_from(this);

        return _clone;

    } //copy_from

    inline function toString():String {

        return 'centroid1: $centroid1 \n
                centroid2: $centroid2 \n
                fraction1: $fraction1 \n
                fraction2: $fraction2 \n
                distance1: $distance1 \n
                distance2: $distance2 \n
                normal: $normal \n
                point: $point
                ';
        
    }
} //RayIntersection