package physics.collision;


import clay.Vector;


	/** simple intersection tests */
class Intersect {


	public static inline var EPSILON:Float = 1.0E-8;

	inline public static function AABBAABB(b1pos:Vector, b1half:Vector, b2pos:Vector, b2half:Vector ):Bool{ // fastest

		if(Math.abs(b1pos.x - b2pos.x) > b1half.x + b2half.x) return false;
		if(Math.abs(b1pos.y - b2pos.y) > b1half.y + b2half.y) return false;

		return true;
	}

	inline public static function AABBAABB2(b1posX:Float, b1posY:Float, b1halfX:Float, b1halfY:Float, b2posX:Float, b2posY:Float, b2halfX:Float, b2halfY:Float):Bool {

		if(Math.abs(b1posX - b2posX) > b1halfX + b2halfX) return false;
		if(Math.abs(b1posY - b2posY) > b1halfY + b2halfY) return false;

		return true;

	}

	inline public static function BoundsBounds(box1min:Vector, box1max:Vector, box2min:Vector, box2max:Vector):Bool{

		if (box1max.x < box2min.x || box1min.x > box2max.x) return false;
		if (box1max.y < box2min.y || box1min.y > box2max.y) return false;

		return true;

	}

	inline public static function BoundsBox(box1min:Vector, box1max:Vector, box2pos:Vector, box2size:Vector):Bool{

		if (box1max.x < (box2pos.x - box2size.x * 0.5) || box1min.x > (box2pos.x + box2size.x * 0.5)) return false;
		if (box1max.y < (box2pos.y - box2size.y * 0.5) || box1min.y > (box2pos.y + box2size.y * 0.5)) return false;

		return true;

	}

	inline public static function BoundsBoxOrigin(box1min:Vector, box1max:Vector, box2pos:Vector, box2size:Vector, box2origin:Vector):Bool{

		var halfX:Float = (box2size.x * 0.5);
		var halfY:Float = (box2size.y * 0.5);

		if (box1max.x < (box2pos.x - halfX - box2origin.x + halfX) || box1min.x > (box2pos.x + halfX - box2origin.x + halfX)) return false;
		if (box1max.y < (box2pos.y - halfY - box2origin.y + halfY) || box1min.y > (box2pos.y + halfY - box2origin.y + halfY)) return false;

		return true;

	}

	inline public static function BoundsAABBOffset(box1min:Vector, box1max:Vector, box2pos:Vector, box2half:Vector, box2offset:Vector):Bool{

		if (box1max.x < (box2pos.x - box2half.x + box2offset.x) || box1min.x > (box2pos.x + box2half.x + box2offset.x)) return false;
		if (box1max.y < (box2pos.y - box2half.y + box2offset.y) || box1min.y > (box2pos.y + box2half.y + box2offset.y)) return false;

		return true;

	}

	inline public static function BoundsAABB(box1min:Vector, box1max:Vector, box2pos:Vector, box2half:Vector):Bool{

		if (box1max.x < (box2pos.x - box2half.x) || box1min.x > (box2pos.x + box2half.x)) return false;
		if (box1max.y < (box2pos.y - box2half.y) || box1min.y > (box2pos.y + box2half.y)) return false;

		return true;

	}

	inline public static function PointAABB(point:Vector, boxPos:Vector, boxHalf:Vector):Bool{

		if(Math.abs(boxPos.x - point.x) > boxHalf.x) return false;
		if(Math.abs(boxPos.y - point.y) > boxHalf.y) return false;

		return true;

	}

	inline public static function PointBounds(point:Vector, boxMin:Vector, boxMax:Vector):Bool{

		if ( point.x < boxMin.x || point.x > boxMax.x ) return false;
		if ( point.y < boxMin.y || point.y > boxMax.y ) return false;

		return true;

	}


    inline public static function CircleCircle( c1pos:Vector, c1r:Float, c2pos:Vector, c2r:Float) : Bool {

        var dx:Float = c1pos.x - c2pos.x;
        var dy:Float = c1pos.y - c2pos.y;
        var r:Float = c1r + c2r;

		return (dx * dx + dy * dy) < (r * r);

    }

	inline public static function CircleAABB(circlePos:Vector, radius:Float, boxPos:Vector, boxHalf:Vector):Bool {

		var cpX:Float = circlePos.x;
		var cpY:Float = circlePos.y;

		var minX:Float = boxPos.x - boxHalf.x;
		var minY:Float = boxPos.y - boxHalf.y;
		var maxX:Float = boxPos.x + boxHalf.x;
		var maxY:Float = boxPos.y + boxHalf.y;

		if (cpX < minX) cpX = minX;
		if (cpX > maxX) cpX = maxX;
		if (cpY < minY) cpY = minY;
		if (cpY > maxY) cpY = maxY;

	    var diffX:Float = circlePos.x - cpX;
	    var diffY:Float = circlePos.y - cpY;

	    return ( diffX * diffX + diffY * diffY < radius * radius );

	}

	inline public static function CircleBox(circlePos:Vector, radius:Float, boxPos:Vector, boxSize:Vector):Bool {

		var cpX:Float = circlePos.x;
		var cpY:Float = circlePos.y;

		var minX:Float = boxPos.x - (boxSize.x * 0.5);
		var minY:Float = boxPos.y - (boxSize.y * 0.5);
		var maxX:Float = boxPos.x + (boxSize.x * 0.5);
		var maxY:Float = boxPos.y + (boxSize.y * 0.5);

		if (cpX < minX) cpX = minX;
		if (cpX > maxX) cpX = maxX;
		if (cpY < minY) cpY = minY;
		if (cpY > maxY) cpY = maxY;

	    var diffX:Float = circlePos.x - cpX;
	    var diffY:Float = circlePos.y - cpY;

	    return ( diffX * diffX + diffY * diffY < radius * radius );

	}

	inline public static function CircleBoxOrigin(circlePos:Vector, radius:Float, boxPos:Vector, boxSize:Vector, boxOrigin:Vector):Bool {

		var cpX:Float = circlePos.x;
		var cpY:Float = circlePos.y;

		var halfX:Float = (boxSize.x * 0.5);
		var halfY:Float = (boxSize.y * 0.5);

		var minX:Float = boxPos.x - halfX - boxOrigin.x + halfX;
		var minY:Float = boxPos.y - halfY - boxOrigin.y + halfY;
		var maxX:Float = boxPos.x + halfX - boxOrigin.x + halfX;
		var maxY:Float = boxPos.y + halfY - boxOrigin.y + halfY;

		if (cpX < minX) cpX = minX;
		if (cpX > maxX) cpX = maxX;
		if (cpY < minY) cpY = minY;
		if (cpY > maxY) cpY = maxY;

	    var diffX:Float = circlePos.x - cpX;
	    var diffY:Float = circlePos.y - cpY;

	    return ( diffX * diffX + diffY * diffY < radius * radius );

	}

	inline public static function CircleAABBOffset(circlePos:Vector, radius:Float, boxPos:Vector, boxHalf:Vector, boxOffset:Vector):Bool {

		var cpX:Float = circlePos.x;
		var cpY:Float = circlePos.y;

		var minX:Float = boxPos.x - boxHalf.x + boxOffset.x;
		var minY:Float = boxPos.y - boxHalf.y + boxOffset.y;
		var maxX:Float = boxPos.x + boxHalf.x + boxOffset.x;
		var maxY:Float = boxPos.y + boxHalf.y + boxOffset.y;

		if (cpX < minX) cpX = minX;
		if (cpX > maxX) cpX = maxX;
		if (cpY < minY) cpY = minY;
		if (cpY > maxY) cpY = maxY;

	    var diffX:Float = circlePos.x - cpX;
	    var diffY:Float = circlePos.y - cpY;

	    return ( diffX * diffX + diffY * diffY < radius * radius );

	}

	inline public static function CircleBounds(circlePos:Vector, radius:Float, bmin:Vector, bmax:Vector):Bool {

		var cpX:Float = circlePos.x;
		var cpY:Float = circlePos.y;

		if (cpX < bmin.x) cpX = bmin.x;
		if (cpX > bmax.x) cpX = bmax.x;
		if (cpY < bmin.y) cpY = bmin.y;
		if (cpY > bmax.y) cpY = bmax.y;

	    var diffX:Float = circlePos.x - cpX;
	    var diffY:Float = circlePos.y - cpY;

	    return ( diffX * diffX + diffY * diffY < radius * radius );

	}

	inline public static function PointCircle(point:Vector, circlePos:Vector, radius:Float):Bool {

		var deltaX:Float = point.x - circlePos.x;
		var deltaY:Float = point.y - circlePos.y;
		var d2:Float = (deltaX * deltaX + deltaY * deltaY);
		var r2:Float = radius * radius;

		return (d2 <= r2);

	}
	

}
