package physics.collision;

import clay.Vector;

import physics.Contact;
// import physics.RaycastHit;

import components.physics.Collider;


class IntersectAABB {


	public static function test(contact:Contact) : Bool {

		var collA:Collider = contact.collider; 
		var collB:Collider = contact.otherCollider; 

		// if(collA.entity.destroyed || collB.entity.destroyed) { // todo
		// 	return false;
		// }

		var dx:Float = collA.position.x - collB.position.x;
		var px:Float = (collB.half.x + collA.half.x) - Math.abs(dx);

		if (px < 0) {
			return false;
		}

		var dy:Float = collA.position.y - collB.position.y;
		var py:Float = (collB.half.y + collA.half.y) - Math.abs(dy);

		if (py < 0) {
			return false;
		}

		contact.reset();

		if (px < py) {
			contact.separation = px;
			contact.normal.x = (dx >= 0) ? 1 : -1;
		} else {
			contact.separation = py;
			contact.normal.y = (dy >= 0) ? 1 : -1;
		}

		// find contact points
		// closestPointOnBox(
		// 	posA.x,
		// 	posA.y,
		// 	posB.x, 
		// 	posB.y, 
		// 	collB.half.x, 
		// 	collB.half.y, 
		// 	contact.point
		//  );

		return true;

	}

	public static function testOneCCD(contact:Contact, dt:Float):Bool{

		var collA:Collider = contact.collider; 
		var collB:Collider = contact.otherCollider; 

		var displacement:Float = collA.velocity.x * dt;
		var slabmin:Float = collB.position.x - collB.half.x - collA.half.x;
		var slabmax:Float = collB.position.x + collB.half.x + collA.half.x;
		var point:Float = collA.position.x;

		var sign:Float;
		var t1:Float;
		var t2:Float;

		contact.reset();

		// x
		if (Math.abs(displacement) < Intersect.EPSILON){
			if((point <= slabmin || point >= slabmax)){
				return false;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / displacement;
			t2 = (slabmax - point) / displacement;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(contact.time > t2 || t1 > contact.timeLast) {
				return false;
			}
			
			if(t1 > contact.time){
				contact.normal.x = sign;
				contact.time = t1;
			}

			if(t2 < contact.timeLast) {
				contact.timeLast = t2;
			}
			
		}

		displacement = collA.velocity.y * dt;
		slabmin = collB.position.y - collB.half.y - collA.half.y;
		slabmax = collB.position.y + collB.half.y + collA.half.y;
		point = collA.position.y;

		// y
		if (Math.abs(displacement) < Intersect.EPSILON){
			if((point <= slabmin || point >= slabmax)){
				return false;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / displacement;
			t2 = (slabmax - point) / displacement;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(contact.time > t2 || t1 > contact.timeLast) {
				return false;
			}
			
			if(t1 > contact.time){
				contact.time = t1;

				contact.normal.x = 0;
				contact.normal.y = sign;
			}

			if(t2 < contact.timeLast) {
				contact.timeLast = t2;
			}
			
		}

		return contact.time > 0.0;
	}

/*
	public static function testCast( b1Pos:Vector, b1Half:Vector, direction:Vector, distance:Float, b2Pos:Vector, b2Half:Vector, c:Collider, ?contact:RaycastHit):RaycastHit{

		contact = (contact == null) ? new RaycastHit() : contact.reset();

		contact.fraction -= Intersect.EPSILON;

		if(!Math.isFinite(distance)){
			distance = 1;
			contact.fractionLast = Math.POSITIVE_INFINITY;
		}

		var displacementX:Float = direction.x * distance;
		var slabmin:Float = b2Pos.x - b2Half.x - b1Half.x;
		var slabmax:Float = b2Pos.x + b2Half.x + b1Half.x;
		var point:Float = b1Pos.x;

		var sign:Float;
		var t1:Float;
		var t2:Float;

		// x
		if (Math.abs(displacementX) < Intersect.EPSILON){
			if((point <= slabmin || point >= slabmax)){
				return null;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / displacementX;
			t2 = (slabmax - point) / displacementX;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(contact.fraction > t2 || t1 > contact.fractionLast) {
				return null;
			}

			if(t1 > contact.fraction){
				contact.fraction = t1;
				contact.normal.x = sign;
			}

			if(t2 < contact.fractionLast) {
				contact.fractionLast = t2;
			}
			
		}


		var displacementY:Float = direction.y * distance;
		slabmin = b2Pos.y - b2Half.y - b1Half.y;
		slabmax = b2Pos.y + b2Half.y + b1Half.y;
		point = b1Pos.y;

		// y
		if (Math.abs(displacementY) < Intersect.EPSILON){
			if((point <= slabmin || point >= slabmax)){
				return null;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / displacementY;
			t2 = (slabmax - point) / displacementY;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(contact.fraction > t2 || t1 > contact.fractionLast) {
				return null;
			}
			

			if(t1 > contact.fraction){
				contact.fraction = t1;
				contact.normal.x = 0;
				contact.normal.y = sign;
			}

			if(t2 < contact.fractionLast) {
				contact.fractionLast = t2;
			}
			
		}

		if(contact.fraction >= 0.0) {

			contact.centroid.x = b1Pos.x;
			contact.centroid.y = b1Pos.y;
			contact.collider = c;

			var dx:Float = displacementX * contact.fraction;
			var dy:Float = displacementY * contact.fraction;

			closestPointOnBox(
				b1Pos.x + dx,
				b1Pos.y + dy,
				b2Pos.x,
				b2Pos.y,
				b2Half.x, 
				b2Half.y, 
				contact.point
			);

			closestPointOnBox(
				b1Pos.x + displacementX * contact.fractionLast,
				b1Pos.y + displacementY * contact.fractionLast,
				b2Pos.x,
				b2Pos.y,
				b2Half.x, 
				b2Half.y, 
				contact.pointOut
			);

			contact.distance = Math.sqrt(dx * dx + dy * dy);

			return contact;
		}

		return null;

	}

	public static function testCastFast( b1Pos:Vector, b1Half:Vector, direction:Vector, distance:Float, b2Pos:Vector, b2Half:Vector):Float{

		var tfirst:Float = -1;
		var tlast:Float = 1;

		if(!Math.isFinite(distance)){
			distance = 1;
			tlast = Math.POSITIVE_INFINITY;
		}

		var displacementX:Float = direction.x * distance;
		var slabmin:Float = b2Pos.x - b2Half.x - b1Half.x;
		var slabmax:Float = b2Pos.x + b2Half.x + b1Half.x;
		var point:Float = b1Pos.x;

		var t1:Float;
		var t2:Float;

		// x
		if (Math.abs(displacementX) < Intersect.EPSILON){
			if((point <= slabmin || point >= slabmax)){
				return -1;
			} 
		} else {

			t1 = (slabmin - point) / displacementX;
			t2 = (slabmax - point) / displacementX;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
			}

			if(tfirst > t2 || t1 > tlast) {
				return -1;
			}

			if(t1 > tfirst){
				tfirst = t1;
			}

			if(t2 < tlast) {
				tlast = t2;
			}
			
		}


		var displacementY:Float = direction.y * distance;
		slabmin = b2Pos.y - b2Half.y - b1Half.y;
		slabmax = b2Pos.y + b2Half.y + b1Half.y;
		point = b1Pos.y;

		// y
		if (Math.abs(displacementY) < Intersect.EPSILON){
			if((point <= slabmin || point >= slabmax)){
				return -1;
			} 
		} else {

			t1 = (slabmin - point) / displacementY;
			t2 = (slabmax - point) / displacementY;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
			}

			if(tfirst > t2 || t1 > tlast) {
				return -1;
			}
			
			if(t1 > tfirst){
				tfirst = t1;
			}
			
		}

		return tfirst;

	}

*/

	public inline static function closestPointOnBox(pposX:Float, pposY:Float, bposX:Float, bposY:Float, bhalfX:Float, bhalfY:Float, out:Vector):Bool {

		var deltaX:Float = pposX - bposX;
		var deltaY:Float = pposY - bposY;

		var inside:Bool = true;

		if (Math.abs(deltaX) > bhalfX)  {
			inside = false;
			deltaX = bhalfX * (deltaX >= 0 ? 1 : -1);
		}

		if (Math.abs(deltaY) > bhalfY) {
			inside = false;
			deltaY = bhalfY * (deltaY >= 0 ? 1 : -1);
		}

		// point was found outside. all good.
		if(!inside){
			out.x = bposX + deltaX;
			out.y = bposY + deltaY;
			return inside;
		}

		// find the mtd (Minimum Translation Distance).
		// calculate the distance of the point form one face along each axes.
		var mtdX:Float = (bhalfX - Math.abs(deltaX)) * (deltaX >= 0 ? 1 : -1);
		var mtdY:Float = (bhalfY - Math.abs(deltaY)) * (deltaY >= 0 ? 1 : -1);

		// Find the minimum of the three.
		if (Math.abs(mtdX) < Math.abs(mtdY)) {
			mtdY = 0.0;
		} else {
			mtdX = 0.0;
		}

		// point on surface
		out.x = pposX + mtdX;
		out.y = pposY + mtdY;

		return inside;

	}
	

}
