package physics.collision;

import clay.Vector;
import clay.utils.Maths;

import components.physics.Collider;
import physics.RaycastHit;


class IntersectLineAABB {


	public static function test( start:Vector, end:Vector, pos:Vector, half:Vector, collider:Collider, ?into:RaycastHit) : RaycastHit {

		into = (into == null) ? new RaycastHit() : into.reset();

		var tlast:Float = 1;

		var deltaX:Float = end.x - start.x;

		var slabmin:Float = pos.x - half.x;
		var slabmax:Float = pos.x + half.x;
		var point:Float = start.x;

		var sign:Float;
		var t1:Float;
		var t2:Float;

		// x
		if (Math.abs(deltaX) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return null;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / deltaX;
			t2 = (slabmax - point) / deltaX;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(into.fraction > t2 || t1 > tlast) {
				return null;
			}

			if(t1 > into.fraction){
				into.fraction = t1;
				into.normal.x = sign;
			}

			if(t2 < tlast) {
				tlast = t2;
			}

		}

		var deltaY:Float = end.y - start.y;
		slabmin = pos.y - half.y;
		slabmax = pos.y + half.y;
		point = start.y;

		// y
		if (Math.abs(deltaY) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return null;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / deltaY;
			t2 = (slabmax - point) / deltaY;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(into.fraction > t2 || t1 > tlast) {
				return null;
			}

			if(t1 > into.fraction){
				into.fraction = t1;
				into.normal.x = 0;
				into.normal.y = sign;
			}

			// if(t2 < tlast) tlast = t2;
			
		}

		if(into.fraction > 0.0){
			into.centroid.x = start.x;
			into.centroid.y = start.y;
			into.collider = collider;


			var dx:Float = deltaX * into.fraction;
			var dy:Float = deltaY * into.fraction;

			into.point.x = start.x + dx;
			into.point.y = start.y + dy;

			into.distance = Math.sqrt(dx * dx + dy * dy);

			return into;
		} 

		return null;

	}

	public static function testFast( start:Vector, end:Vector, pos:Vector, half:Vector ) : Float {

		var tfirst:Float = 0;
		var tlast:Float = 1;

		var deltaX:Float = end.x - start.x;

		var slabmin:Float = pos.x - half.x;
		var slabmax:Float = pos.x + half.x;
		var point:Float = start.x;

		var t1:Float;
		var t2:Float;

		// x
		if (Math.abs(deltaX) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return 0;
			} 
		} else {

			t1 = (slabmin - point) / deltaX;
			t2 = (slabmax - point) / deltaX;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
			}

			if(tfirst > t2 || t1 > tlast) {
				return 0;
			}

			if(t1 > tfirst){
				tfirst = t1;
			}

			if(t2 < tlast) {
				tlast = t2;
			}

		}

		var deltaY:Float = end.y - start.y;
		slabmin = pos.y - half.y;
		slabmax = pos.y + half.y;
		point = start.y;

		// y
		if (Math.abs(deltaY) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return 0;
			} 
		} else {

			t1 = (slabmin - point) / deltaY;
			t2 = (slabmax - point) / deltaY;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
			}

			if(tfirst > t2 || t1 > tlast) {
				return 0;
			}

			if(t1 > tfirst){
				tfirst = t1;
			}
			
		}

		return tfirst;

	}


}
