package physics.collision;

import clay.utils.Maths;
import clay.Vector;

import components.physics.Collider;
import physics.RaycastHit;


class IntersectRayAABB {


	public static function test( origin:Vector, direction:Vector, distance:Float, bPos:Vector, bHalf:Vector, collider:Collider, ?into:RaycastHit) : RaycastHit {

		into = (into == null) ? new RaycastHit() : into.reset();

		var tlast:Float = 1;
		
		if(!Math.isFinite(distance)){
			distance = 1;
			tlast = Math.POSITIVE_INFINITY;
		}

		var deltaX:Float = direction.x * distance;

		var slabmin:Float = bPos.x - bHalf.x;
		var slabmax:Float = bPos.x + bHalf.x;
		var point:Float = origin.x;

		var sign:Float;
		var t1:Float;
		var t2:Float;

		// x
		if (Math.abs(deltaX) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return null;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / deltaX;
			t2 = (slabmax - point) / deltaX;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(into.fraction > t2 || t1 > tlast) {
				return null;
			}

			if(t1 > into.fraction){
				into.fraction = t1;
				into.normal.x = sign;
			}

			if(t2 < tlast) {
				tlast = t2;
			}
			
		}

		var deltaY:Float = direction.y * distance;
		slabmin = bPos.y - bHalf.y;
		slabmax = bPos.y + bHalf.y;
		point = origin.y;

		// y
		if (Math.abs(deltaY) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return null;
			} 
		} else {
			sign = -1;

			t1 = (slabmin - point) / deltaY;
			t2 = (slabmax - point) / deltaY;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
				sign = 1;
			}

			if(into.fraction > t2 || t1 > tlast) {
				return null;
			}

			if(t1 > into.fraction){
				into.fraction = t1;
				into.normal.x = 0;
				into.normal.y = sign;
			}

			// if(t2 < tlast) tlast = t2;
			
		}

		if(into.fraction > 0.0){
			into.centroid.x = origin.x;
			into.centroid.y = origin.y;
			into.collider = collider;

			var dx:Float = deltaX * into.fraction;
			var dy:Float = deltaY * into.fraction;

			into.point.x = origin.x + dx;
			into.point.y = origin.y + dy;

			into.distance = Math.sqrt(dx * dx + dy * dy);

			return into;
		} 

		return null;

	}
	
	public static function testFast( origin:Vector, direction:Vector, distance:Float, bPos:Vector, bHalf:Vector) : Float {

		var tfirst:Float = 0;
		var tlast:Float = 1;
		
		if(!Math.isFinite(distance)){
			distance = 1;
			tlast = Math.POSITIVE_INFINITY;
		}

		var deltaX:Float = direction.x * distance;

		var slabmin:Float = bPos.x - bHalf.x;
		var slabmax:Float = bPos.x + bHalf.x;
		var point:Float = origin.x;

		var t1:Float;
		var t2:Float;

		// x
		if (Math.abs(deltaX) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return 0;
			} 
		} else {

			t1 = (slabmin - point) / deltaX;
			t2 = (slabmax - point) / deltaX;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
			}

			if(tfirst > t2 || t1 > tlast) {
				return 0;
			}

			if(t1 > tfirst){
				tfirst = t1;
			}

			if(t2 < tlast) {
				tlast = t2;
			}
			
		}

		var deltaY:Float = direction.y * distance;
		slabmin = bPos.y - bHalf.y;
		slabmax = bPos.y + bHalf.y;
		point = origin.y;

		// y
		if (Math.abs(deltaY) < Intersect.EPSILON){
			if((point < slabmin || point > slabmax)){
				return 0;
			} 
		} else {

			t1 = (slabmin - point) / deltaY;
			t2 = (slabmax - point) / deltaY;

			if(t1 > t2){
				var _tmp:Float = t1;
				t1 = t2;
				t2 = _tmp;
			}

			if(tfirst > t2 || t1 > tlast) {
				return 0;
			}

			if(t1 > tfirst){
				tfirst = t1;
			}

		}

		return tfirst;

	}


}
