package physics.collision;

import clay.utils.Maths;
import clay.Vector;

import physics.collision.data.RayIntersection;


class IntersectRayRay {


    public static function test( r1pos:Vector, r1dir:Vector, r1dist:Float, r2pos:Vector, r2dir:Vector, r2dist:Float, ?into:RayIntersection ) : RayIntersection {

        var end1:Float = 1;
		if(!Math.isFinite(r1dist)){
			r1dist = 1;
        	end1 = Math.POSITIVE_INFINITY;
		}

        var end2:Float = 1;
		if(!Math.isFinite(r2dist)){
			r2dist = 1;
        	end2 = Math.POSITIVE_INFINITY;
		}

		var delta1X:Float = r1dir.x * r1dist;
		var delta1Y:Float = r1dir.y * r1dist;

		var delta2X:Float = r2dir.x * r2dist;
		var delta2Y:Float = r2dir.y * r2dist;

        var diffX:Float = r1pos.x - r2pos.x;
        var diffY:Float = r1pos.y - r2pos.y;

        var ud:Float = delta2Y * delta1X - delta2X * delta1Y;
        
        if(ud == 0.0) {
            return null;
        }

        var u1:Float = (delta2X * diffY - delta2Y * diffX) / ud;
        var u2:Float = (delta1X * diffY - delta1Y * diffX) / ud;

        if ((u1 > 0.0 && u1 <= end1) && (u2 > 0.0 && u2 <= end2)) {
            into = (into == null) ? new RayIntersection() : into.reset();

                into.centroid1.x = r1pos.x;
                into.centroid1.y = r1pos.y;

                into.centroid2.x = r2pos.x;
                into.centroid2.y = r2pos.y;

                into.fraction1 = u1;
                into.fraction2 = u2;

            	var dx:Float = delta1X * u1;
				var dy:Float = delta1Y * u1;

                into.point.x = r1pos.x + dx;
	        	into.point.y = r1pos.y + dy;

            	into.distance1 = Math.sqrt(dx * dx + dy * dy);

				dx = delta2X * u2;
				dy = delta2Y * u2;

            	into.distance2 = Math.sqrt(dx * dx + dy * dy);

            return into;
        }

        return null;

    }


}
